@@ -579,7 +579,7 @@ def plot_id_bar(id_df, nsv=None, logger=None, **kwargs):
     else:
         ticks = np.arange(1, nsv + 1, int((nsv + 1) / 30))
 
-    cb = plt.colorbar(sm)
+    cb = plt.colorbar(sm, ax=ax)
     cb.set_ticks(ticks)
 
     logger.log("plot id bar")
@@ -1384,6 +1384,68 @@ def ensemble_res_1to1(
         plt.show()
 
     """
+    def _get_plotlims(oen, ben, obsnames):
+        if not isinstance(oen, dict):
+            oen = {'g': oen.loc[:, obsnames]}
+        if not isinstance(ben, dict):
+            ben = {'g': ben.get(obsnames)}
+        outofrange = False
+        # work back from crazy values
+        oemin = 1e32
+        oemeanmin = 1e32
+        oemax = -1e32
+        oemeanmax = -1e32
+        bemin = 1e32
+        bemeanmin = 1e32
+        bemax = -1e32
+        bemeanmax = -1e32
+        for _, oeni in oen.items():  # loop over ensembles
+            oeni = oeni.loc[:, obsnames]  # slice group obs
+            oemin = np.min([oemin, oeni.min().min()])
+            oemax = np.max([oemax, oeni.max().max()])
+            # get min and max of mean sim vals
+            # (incase we want plot to ignore extremes)
+            oemeanmin = np.min([oemeanmin, oeni.mean().min()])
+            oemeanmax = np.max([oemeanmax, oeni.mean().max()])
+        for _, beni in ben.items():  # same with base ensemble/obsval
+            # work with either ensemble or obsval series
+            beni = beni.get(obsnames)
+            bemin = np.min([bemin, beni.min().min()])
+            bemax = np.max([bemax, beni.max().max()])
+            bemeanmin = np.min([bemeanmin, beni.mean().min()])
+            bemeanmax = np.max([bemeanmax, beni.mean().max()])
+        # get base ensemble range
+        berange = bemax-bemin
+        if berange == 0.:  # only one obs in group (probs)
+            berange = bemeanmax * 1.1  # expand a little
+        # add buffer to obs endpoints
+        bemin = bemin - (berange*0.05)
+        bemax = bemax + (berange*0.05)
+        if oemax < bemin:  # sim well below obs
+            oemin = oemeanmin  # set min to mean min
+            # (sim captured but not extremes)
+            outofrange = True
+        if oemin > bemax:  # sim well above obs
+            oemax = oemeanmax
+            outofrange = True
+        oerange = oemax - oemin
+        if bemax > oemax + (0.1*oerange):  # obs max well above sim
+            if not outofrange:  # but sim still in range
+                # zoom to sim
+                bemax = oemax + (0.1*oerange)
+            else:  # use obs mean max
+                bemax = bemeanmax
+        if bemin < oemin - (0.1 * oerange):  # obs min well below sim
+            if not outofrange:  # but sim still in range
+                # zoom to sim
+                bemin = oemin - (0.1 * oerange)
+            else:
+                bemin = bemeanmin
+        pmin = np.min([oemin, bemin])
+        pmax = np.max([oemax, bemax])
+        return pmin, pmax
+
+
     if logger is None:
         logger = Logger("Default_Loggger.log", echo=False)
     logger.log("plot res_1to1")
@@ -1412,17 +1474,13 @@ def ensemble_res_1to1(
             ),
             ha="center",
         )
-    # if plot_hexbin:
-    #    pdfname = pst.filename.replace(".pst", ".1to1.hexbin.pdf")
-    # else:
-    #    pdfname = pst.filename.replace(".pst", ".1to1.pdf")
+
     figs = []
     ax_count = 0
     for g, names in grouper.items():
         logger.log("plotting 1to1 for {0}".format(g))
         # control file observation for group
         obs_g = obs.loc[names, :]
-        logger.statement("using control file obsvals to calculate residuals")
         # normally only look a non-zero weighted obs
         if "include_zero" not in kwargs or kwargs["include_zero"] is False:
             obs_g = obs_g.loc[obs_g.weight > 0, :]
@@ -1434,83 +1492,47 @@ def ensemble_res_1to1(
         if ax_count % (nr * nc) == 0:
             if ax_count > 0:
                 plt.tight_layout()
-            # pdf.savefig()
-            # plt.close(fig)
             figs.append(fig)
             fig = plt.figure(figsize=figsize)
             axes = _get_page_axes()
             ax_count = 0
         ax = axes[ax_count]
 
-        # min and max for actual observations (xaxis) from control file
-        obx = obs_g.obsval.max()  # original base obsval max
-        obn = obs_g.obsval.min()  # original base obsval min
         if base_ensemble is None:
             # if obs not defined by obs+noise ensemble,
             # use min and max for obsval from control file
-            bx = obx
-            bn = obn
+            pmin, pmax = _get_plotlims(ensembles, obs_g.obsval, obs_g.obsnme)
         else:
             # if obs defined by obs+noise use obs+noise min and max
-            # ben = base_ensemble["r"]
-            # ben = ben.loc[:, ben.columns.intersection(names)]
-            bn = 1e32  # ben.min().min()
-            bx = -1e32  # ben.max().max()
+            pmin, pmax = _get_plotlims(ensembles, base_ensemble, obs_g.obsnme)
             obs_gg = obs_g.sort_values(by="obsval")
             for c, en in base_ensemble.items():
                 en_g = en.loc[:, obs_gg.obsnme]
-                ex = en_g.max()
-                en = en_g.min()
+                emx = en_g.max()
+                emn = en_g.min()
                 # update y min and max for obs+noise ensembles
-                bn = np.min([en.min(), bn])
-                bx = np.max([ex.max(), bx])
-                # [ax.plot([ov, ov], [een, eex], color=c,alpha=0.3) for ov, een, eex in zip(obs_g.obsval.values, en.values, ex.values)]
-                ax.fill_between(obs_gg.obsval, en, ex, facecolor=c, alpha=0.2, zorder=2)
-        # ax.scatter([obs_g.sim], [obs_g.obsval], marker='.', s=10, color='b')
-        # collector for mins and max
-        omn = []
-        omx = []
+                if len(obs_gg.obsval) > 1:
+                    emx = np.zeros(obs_gg.shape[0]) + emx
+                    emn = np.zeros(obs_gg.shape[0]) + emn
+                    ax.fill_between(obs_gg.obsval.values, emn.values, emx.values,
+                                    facecolor=c, alpha=0.2, zorder=2)
+                else:
+                    ax.plot([obs_gg.obsval.values, obs_gg.obsval.values], [emn, emx], color=c, alpha=0.2, zorder=2)
         for c, en in ensembles.items():
             en_g = en.loc[:, obs_g.obsnme]
             # output mins and maxs
-            ex = en_g.max()
-            en = en_g.min()
-            omn.append(en)
-            omx.append(ex)
+            emx = en_g.max()
+            emn = en_g.min()
             [
                 ax.plot([ov, ov], [een, eex], color=c, zorder=1)
-                for ov, een, eex in zip(obs_g.obsval.values, en.values, ex.values)
+                for ov, een, eex in zip(obs_g.obsval.values, emn.values, emx.values)
             ]
+        ax.plot([pmin, pmax], [pmin, pmax], "k--", lw=1.0, zorder=3)
+        xlim = (pmin, pmax)
+        ax.set_xlim(pmin, pmax)
+        ax.set_ylim(pmin, pmax)
 
-        omn = pd.concat(omn).min()
-        omx = pd.concat(omx).max()
-        # focus on obs(+noise)
-        # need to make sure all obsval are captured (obn, obx)
-        # but helpful if not zoomed out too far
-        rng = bx - bn
-        mpnt = rng / 2
-        if omn < bn:  # if the output ensemble mins extend below obs+noise
-            mn = bn - 0.01 * rng  # focus on obs+noise? -- will capture obsval
-        elif (
-            1.1 * (mpnt - bn) <= mpnt - omn
-        ):  # if min of output en is close to obs+noise
-            mn = mpnt - (1.1 * (mpnt - omn))  # expand from model output a bit
-        else:
-            mn = omn - 0.02 * (omx - omn)  # focus on model output
-        if omx > bx:  # if the output ensemble max is above the obs+noise max
-            mx = bx + 0.01 * rng  # focus on the obs+noise max
-        elif (
-            1.1 * (bx - mpnt) <= omx - mpnt
-        ):  # if max of output en is close to obs_nois
-            mx = mpnt + (1.1 * (omx - mpnt))  # expand from model output a bit
-        else:
-            mx = omx + 0.02 * (omx - omn)  # focus on model output
-        ax.plot([mn, mx], [mn, mx], "k--", lw=1.0, zorder=3)
-        xlim = (mn, mx)
-        ax.set_xlim(mn, mx)
-        ax.set_ylim(mn, mx)
-
-        if mx > 1.0e5:
+        if max(np.abs(xlim)) > 1.0e5:
             ax.xaxis.set_major_formatter(matplotlib.ticker.FormatStrFormatter("%1.0e"))
             ax.yaxis.set_major_formatter(matplotlib.ticker.FormatStrFormatter("%1.0e"))
         ax.grid()
@@ -1531,53 +1553,54 @@ def ensemble_res_1to1(
 
         if base_ensemble is not None:
             obs_gg = obs_g.sort_values(by="obsval")
-            bn = 1e32  # ben.min().min()
-            bx = -1e32  # ben.max().max()
             for c, en in base_ensemble.items():
                 en_g = en.loc[:, obs_gg.obsnme].subtract(obs_gg.obsval)
-                ex = en_g.max()
-                en = en_g.min()
-                # update y min and max for obs+noise ensembles
-                bn = np.min([en.min(), bn])
-                bx = np.max([ex.max(), bx])
-                # [ax.plot([ov, ov], [een, eex], color=c,alpha=0.3) for ov, een, eex in zip(obs_g.obsval.values, en.values, ex.values)]
-                ax.fill_between(obs_gg.obsval, en, ex, facecolor=c, alpha=0.2, zorder=2)
+                emx = en_g.max()
+                emn = en_g.min()
+                if len(obs_gg.obsval) > 1:
+                    emx = np.zeros(obs_gg.shape[0]) + emx
+                    emn = np.zeros(obs_gg.shape[0]) + emn
+                    ax.fill_between(obs_gg.obsval.values, emn.values, emx.values,
+                                    facecolor=c, alpha=0.2, zorder=2)
+                else:
+                    # [ax.plot([ov, ov], [een, eex], color=c,alpha=0.3) for ov, een, eex in zip(obs_g.obsval.values, en.values, ex.values)]
+                    ax.plot([obs_gg.obsval.values, obs_gg.obsval.values], [emn, emx], color=c,
+                            alpha=0.2, zorder=2)
         omn = []
         omx = []
         for c, en in ensembles.items():
             en_g = en.loc[:, obs_g.obsnme].subtract(obs_g.obsval, axis=1)
-            ex = en_g.max()
-            en = en_g.min()
-            omn.append(en)
-            omx.append(ex)
+            emx = en_g.max()
+            emn = en_g.min()
+            omn.append(emn)
+            omx.append(emx)
             [
                 ax.plot([ov, ov], [een, eex], color=c, zorder=1)
-                for ov, een, eex in zip(obs_g.obsval.values, en.values, ex.values)
+                for ov, een, eex in zip(obs_g.obsval.values, emn.values, emx.values)
             ]
 
         omn = pd.concat(omn).min()
         omx = pd.concat(omx).max()
-        # always focus on outputs
-        # -> if obs(+ noise) is broader, focus on sim out
-        mn = omn
-        mx = omx
-        ax.set_ylim(mn, mx)
-
-        # if base_ensemble is not None:
-        #     if base_ensemble is not None:
-        #         for c, en in base_ensemble.items():
-        #             en_g = en.loc[:, obs_g.obsnme].subtract(obs_g.obsval,axis=1)
-        #             ex = en_g.max()
-        #             en = en_g.min()
-        #             [ax.plot([ov, ov], [een, eex], color=c, alpha=0.3) for ov, een, eex in
-        #              zip(obs_g.obsval.values, en.values, ex.values)]
-        ylim = ax.get_ylim()
-        mx = max(np.abs(ylim[0]), np.abs(ylim[1]))  # ensure symmetric about y=0
+        mx = max(np.abs(omn), np.abs(omx))  # ensure symmetric about y=0
         if obs_g.shape[0] == 1:
-            mx *= 1.1
+            mx *= 1.05
         else:
             mx *= 1.02
-        ax.set_ylim(-mx, mx)
+        if np.sign(omn) == np.sign(omx):
+            # allow y axis asymm if all above or below
+            mn = np.min([0, np.sign(omn) * mx])
+            mx = np.max([0, np.sign(omn) * mx])
+        else:
+            mn = -mx
+        ax.set_ylim(mn, mx)
+        bmin = obs_g.obsval.values.min()
+        bmax = obs_g.obsval.values.max()
+        brange = (bmax - bmin)
+        if brange == 0.:
+            brange = obs_g.obsval.values.mean()
+        bmin = bmin - 0.1*brange
+        bmax = bmax + 0.1*brange
+        xlim = (bmin, bmax)
         # show a zero residuals line
         ax.plot(xlim, [0, 0], "k--", lw=1.0, zorder=3)
 
@@ -1604,8 +1627,6 @@ def ensemble_res_1to1(
         axes[a].set_xticks([])
 
     plt.tight_layout()
-    # pdf.savefig()
-    # plt.close(fig)
     figs.append(fig)
     if filename is not None:
         plt.tight_layout()
