@@ -331,11 +331,13 @@ class SpecSim2d(object):
 
     Example::
 
-        v = pyemu.utils.geostats.ExpVario(a=1000,contribution=1.0)
+        v = pyemu.utils.geostats.ExpVario(a=100,contribution=1.0)
         gs = pyemu.utils.geostats.GeoStruct(variograms=v,nugget=0.5)
-        delx,dely = np.arange(100), np.arrange(100)
+        delx,dely = np.ones(150), np.ones(50)
         ss = pyemu.utils.geostats.SpecSim2d(delx,dely,gs)
-        arrays = ss.draw(num_reals=100)
+        arr = np.squeeze(ss.draw_arrays(num_reals=1))*.05 + .08
+        plt.imshow(arr)
+        plt.colorbar(shrink=.40)
 
     """
 
@@ -522,8 +524,7 @@ class SpecSim2d(object):
             raise Exception(
                 "SpecSim2D grid_par_ensemble_helper() error: only a single variogram can be used..."
             )
-        gr_df.loc[:, "i"] = gr_df.i.apply(np.int64)
-        gr_df.loc[:, "j"] = gr_df.j.apply(np.int64)
+        gr_df.loc[:, ["i", "j"]] = gr_df[["i", "j"]].astype(int)
 
         # scale the total contrib
         org_var = self.geostruct.variograms[0].contribution
@@ -940,10 +941,16 @@ class OrdinaryKrige(object):
             )
 
         if var_filename is not None:
-            arr = (
-                np.zeros((self.spatial_reference.nrow, self.spatial_reference.ncol))
+            if self.spatial_reference.grid_type=='vertex':
+                arr = (
+                np.zeros((self.spatial_reference.ncpl, 1))
                 - 1.0e30
-            )
+                )
+            else:
+                arr = (
+                    np.zeros((self.spatial_reference.nrow, self.spatial_reference.ncol))
+                    - 1.0e30
+                )
 
         # the simple case of no zone array: ignore point_data zones
         if zone_array is None:
@@ -964,7 +971,10 @@ class OrdinaryKrige(object):
                 np.savetxt(var_filename, arr, fmt="%15.6E")
 
         if zone_array is not None:
-            assert zone_array.shape == x.shape
+            if self.spatial_reference.grid_type=='vertex':
+                assert zone_array.shape[0] == x.shape[0]
+            else:
+                assert zone_array.shape == x.shape
             if "zone" not in self.point_data.columns:
                 warnings.warn(
                     "'zone' columns not in point_data, assigning generic zone",
@@ -983,8 +993,13 @@ class OrdinaryKrige(object):
                     )
                     continue
                 # cutting list of cell positions to just in zone
-                xzone = x[zone_array == pt_data_zone].copy()
-                yzone = y[zone_array == pt_data_zone].copy()
+                if spatial_reference.grid_type == "vertex":
+                    xzone = x[(zone_array == pt_data_zone).ravel()].copy()
+                    yzone = y[(zone_array == pt_data_zone).ravel()].copy()
+                else:
+                    xzone = x[zone_array == pt_data_zone].copy()
+                    yzone = y[zone_array == pt_data_zone].copy()
+                
                 idx = np.arange(
                     len(zone_array.ravel())
                 )[(zone_array == pt_data_zone).ravel()]
@@ -1016,6 +1031,8 @@ class OrdinaryKrige(object):
                     fulldf = fulldf.reset_index()
                     a = fulldf.err_var.values.reshape(x.shape)
                     na_idx = np.isfinite(a.astype(float))
+                    if len(a.shape)==1:
+                        a = np.reshape(a, (a.shape[0], 1))
                     arr[na_idx] = a[na_idx]
             if self.interp_data is None or self.interp_data.dropna().shape[0] == 0:
                 raise Exception("no interpolation took place...something is wrong")
@@ -1450,7 +1467,7 @@ class OrdinaryKrige(object):
             if self.interp_data is None:
                 self.interp_data = df
             else:
-                self.interp_data = self.interp_data.append(df)
+                self.interp_data = pd.concat([self.interp_data, df])
         # correct for negative kriging factors, if requested
         if remove_negative_factors == True:
             self._remove_neg_factors()
@@ -1605,6 +1622,9 @@ class OrdinaryKrige(object):
             nrow = 1
             if ncol < self.interp_data.shape[0]:
                 raise Exception("something is wrong")
+        elif self.spatial_reference.grid_type=='vertex':
+            nrow = self.spatial_reference.ncpl
+            ncol = 1
         else:
             nrow = self.spatial_reference.nrow
             ncol = self.spatial_reference.ncol
