@@ -6,6 +6,8 @@ import numpy as np
 import pandas as pd
 import warnings
 
+
+
 pd.options.display.max_colwidth = 100
 from pyemu.pst.pst_utils import SFMT, IFMT, FFMT, pst_config
 from pyemu.utils.helpers import run, _write_df_tpl
@@ -33,7 +35,6 @@ def setup_pilotpoints_grid(
     pp_dir=".",
     tpl_dir=".",
     shapename="pp.shp",
-    longnames=False,
     pp_filename_dict={},
 ):
     """setup a regularly-spaced (gridded) pilot point parameterization
@@ -59,7 +60,6 @@ def setup_pilotpoints_grid(
         tpl_dir (`str`, optional): directory to write pilot point template file to.  Default is '.'
         shapename (`str`, optional): name of shapefile to write that contains pilot
             point information. Default is "pp.shp"
-        longnames (`bool`): flag to use parameter names longer than 12 chars.  Default is False
         pp_filename_dict (`dict`): optional dict of prefix-pp filename pairs.  prefix values must
             match the values in `prefix_dict`.  If None, then pp filenames are based on the
             key values in `prefix_dict`.  Default is None
@@ -130,6 +130,12 @@ def setup_pilotpoints_grid(
             )
     start = int(float(every_n_cell) / 2.0)
 
+    if sr.grid_type=='vertex':
+        if len(xcentergrid.shape)==1:
+            xcentergrid = np.reshape(xcentergrid, (xcentergrid.shape[0], 1))
+            ycentergrid = np.reshape(ycentergrid, (ycentergrid.shape[0], 1))
+
+
     # fix for x-section models
     if xcentergrid.shape[0] == 1:
         start_row = 0
@@ -169,7 +175,10 @@ def setup_pilotpoints_grid(
     par_info = []
     pp_files, tpl_files = [], []
     pp_names = copy.copy(PP_NAMES)
-    pp_names.extend(["k", "i", "j"])
+    if sr.grid_type == "vertex":
+        pp_names.extend(["k"])
+    else:
+        pp_names.extend(["k", "i", "j"])
 
     if not np.all([isinstance(v, dict) for v in ibound.values()]):
         ibound = {"general_zn": ibound}
@@ -177,50 +186,81 @@ def setup_pilotpoints_grid(
     par_keys.sort()
     for par in par_keys:
         for k in range(len(ibound[par])):
+            # skip this layer if not in prefix_dict
+            if k not in prefix_dict.keys():
+                continue
+
             pp_df = None
             ib = ibound[par][k]
             assert (
                 ib.shape == xcentergrid.shape
             ), "ib.shape != xcentergrid.shape for k {0}".format(k)
-            pp_count = 0
-            # skip this layer if not in prefix_dict
-            if k not in prefix_dict.keys():
-                continue
-            # cycle through rows and cols
-            for i in range(start_row, ib.shape[0] - start_row, every_n_cell):
-                for j in range(start_col, ib.shape[1] - start_col, every_n_cell):
-                    # skip if this is an inactive cell
-                    if ib[i, j] <= 0:  # this will account for MF6 style ibound as well
-                        continue
 
-                    # get the attributes we need
-                    x = xcentergrid[i, j]
-                    y = ycentergrid[i, j]
-                    name = "pp_{0:04d}".format(pp_count)
+            pp_count = 0
+            if sr.grid_type == "vertex":
+                spacing=every_n_cell
+                
+                for zone in np.unique(ib):
+                    # escape <zero idomain values
+                    if zone <= 0:
+                        continue
+                    ppoint_xys = get_zoned_ppoints_for_vertexgrid(spacing, ib, sr, zone_number=zone)
+
                     parval1 = 1.0
 
-                    # decide what to use as the zone
-                    zone = 1
-                    if use_ibound_zones:
-                        zone = ib[i, j]
-                    # stick this pilot point into a dataframe container
+                    for pp in ppoint_xys:
+                        name = "pp_{0:04d}".format(pp_count)
+                        x,y = pp[0], pp[-1]
+                        if pp_df is None:
+                            data = {
+                                "name": name,
+                                "x": x,
+                                "y": y,
+                                "zone": zone,  # if use_ibound_zones is False this will always be 1
+                                "parval1": parval1,
+                                "k": k,
+                            }
+                            pp_df = pd.DataFrame(data=data, index=[0], columns=pp_names)
+                        else:
+                            data = [name, x, y, zone, parval1, k,]
+                            pp_df.loc[pp_count, :] = data
+                            pp_count+=1
 
-                    if pp_df is None:
-                        data = {
-                            "name": name,
-                            "x": x,
-                            "y": y,
-                            "zone": zone, # if use_ibound_zones is False this will always be 1
-                            "parval1": parval1,
-                            "k": k,
-                            "i": i,
-                            "j": j,
-                        }
-                        pp_df = pd.DataFrame(data=data, index=[0], columns=pp_names)
-                    else:
-                        data = [name, x, y, zone, parval1, k, i, j]
-                        pp_df.loc[pp_count, :] = data
-                    pp_count += 1
+            else:
+                # cycle through rows and cols
+                for i in range(start_row, ib.shape[0] - start_row, every_n_cell):
+                    for j in range(start_col, ib.shape[1] - start_col, every_n_cell):
+                        # skip if this is an inactive cell
+                        if ib[i, j] <= 0:  # this will account for MF6 style ibound as well
+                            continue
+                        # get the attributes we need
+                        x = xcentergrid[i, j]
+                        y = ycentergrid[i, j]
+                        name = "pp_{0:04d}".format(pp_count)
+                        parval1 = 1.0
+
+                        # decide what to use as the zone
+                        zone = 1
+                        if use_ibound_zones:
+                            zone = ib[i, j]
+                        # stick this pilot point into a dataframe container
+
+                        if pp_df is None:
+                            data = {
+                                "name": name,
+                                "x": x,
+                                "y": y,
+                                "zone": zone,  # if use_ibound_zones is False this will always be 1
+                                "parval1": parval1,
+                                "k": k,
+                                "i": i,
+                                "j": j,
+                            }
+                            pp_df = pd.DataFrame(data=data, index=[0], columns=pp_names)
+                        else:
+                            data = [name, x, y, zone, parval1, k, i, j]
+                            pp_df.loc[pp_count, :] = data
+                        pp_count += 1
             # if we found some acceptable locs...
             if pp_df is not None:
                 for prefix in prefix_dict[k]:
@@ -239,8 +279,8 @@ def setup_pilotpoints_grid(
 
                         tpl_filename = os.path.join(tpl_dir, base_filename + ".tpl")
                         # write the tpl file
-                        pilot_points_to_tpl(
-                            pp_df, tpl_filename, name_prefix=prefix, longnames=longnames
+                        pp_df = pilot_points_to_tpl(
+                            pp_df, tpl_filename, name_prefix=prefix,
                         )
                         pp_df.loc[:, "tpl_filename"] = tpl_filename
                         pp_df.loc[:, "pp_filename"] = pp_filename
@@ -252,12 +292,14 @@ def setup_pilotpoints_grid(
                         tpl_files.append(tpl_filename)
 
     par_info = pd.concat(par_info)
-    for field in ["k", "i", "j"]:
-        par_info.loc[:, field] = par_info.loc[:, field].apply(np.int64)
-    for key, default in pst_config["par_defaults"].items():
-        if key in par_info.columns:
-            continue
-        par_info.loc[:, key] = default
+    if sr.grid_type == "vertex":
+        fields = ["k", "zone"]
+    else:
+        fields = ["k", "i", "j", "zone"]
+    par_info = par_info.astype({f: int for f in fields}, errors='ignore')
+    defaults = pd.DataFrame(pst_config["par_defaults"], index=par_info.index)
+    missingcols = defaults.columns.difference(par_info.columns)
+    par_info.loc[:, missingcols] = defaults
 
     if shapename is not None:
         try:
@@ -271,17 +313,26 @@ def setup_pilotpoints_grid(
             shp = shapefile.Writer(target=shapename, shapeType=shapefile.POINT)
         except:
             shp = shapefile.Writer(shapeType=shapefile.POINT)
-        for name, dtype in par_info.dtypes.iteritems():
+        for name, dtype in par_info.dtypes.items():
             if dtype == object:
                 shp.field(name=name, fieldType="C", size=50)
-            elif dtype in [int, np.int64, np.int32]:
+            elif dtype in [int]:#, np.int64, np.int32]:
                 shp.field(name=name, fieldType="N", size=50, decimal=0)
             elif dtype in [float, np.float32, np.float64]:
                 shp.field(name=name, fieldType="N", size=50, decimal=10)
             else:
-                raise Exception(
-                    "unrecognized field type in par_info:{0}:{1}".format(name, dtype)
-                )
+                try:
+                    if dtype in [np.int64, np.int32]:
+                        shp.field(name=name, fieldType="N", size=50, decimal=0)
+                    else:
+                        raise Exception(
+                            "unrecognized field type in par_info:{0}:{1}".format(name, dtype)
+                        )
+
+                except Exception as e:
+                    raise Exception(
+                        "unrecognized field type in par_info:{0}:{1}".format(name, dtype)
+                    )
 
         # some pandas awesomeness..
         par_info.apply(lambda x: shp.point(x.x, x.y), axis=1)
@@ -290,9 +341,10 @@ def setup_pilotpoints_grid(
             shp.save(shapename)
         except:
             shp.close()
-
+        shp.close()
         shp = shapefile.Reader(shapename)
         assert shp.numRecords == par_info.shape[0]
+        shp.close()
     return par_info
 
 
@@ -446,14 +498,23 @@ def write_pp_shapfile(pp_df, shapename=None):
     for name, dtype in dfs[0].dtypes.iteritems():
         if dtype == object:
             shp.field(name=name, fieldType="C", size=50)
-        elif dtype in [int, np.int, np.int64, np.int32]:
+        elif dtype in [int]:#, np.int, np.int64, np.int32]:
             shp.field(name=name, fieldType="N", size=50, decimal=0)
         elif dtype in [float, np.float32, np.float32]:
             shp.field(name=name, fieldType="N", size=50, decimal=8)
         else:
-            raise Exception(
-                "unrecognized field type in pp_df:{0}:{1}".format(name, dtype)
-            )
+            try:
+                if dtype in [np.int64, np.int32]:
+                    shp.field(name=name, fieldType="N", size=50, decimal=0)
+                else:
+                    raise Exception(
+                        "unrecognized field type in par_info:{0}:{1}".format(name, dtype)
+                    )
+
+            except Exception as e:
+                raise Exception(
+                    "unrecognized field type in par_info:{0}:{1}".format(name, dtype)
+                )
 
     # some pandas awesomeness..
     for df in dfs:
@@ -490,7 +551,7 @@ def write_pp_file(filename, pp_df):
         )
 
 
-def pilot_points_to_tpl(pp_file, tpl_file=None, name_prefix=None, longnames=False):
+def pilot_points_to_tpl(pp_file, tpl_file=None, name_prefix=None):
     """write a template file for a pilot points file
 
     Args:
@@ -502,8 +563,6 @@ def pilot_points_to_tpl(pp_file, tpl_file=None, name_prefix=None, longnames=Fals
             pilot point parameters will be named "hk_0001","hk_0002", etc.
             If None, parameter names from `pp_df.name` are used.
             Default is None.
-        longnames (`bool`): flag to use parameter names longer than 12 chars.
-            Default is False
 
     Returns:
         `pandas.DataFrame`: a dataframe with pilot point information
@@ -512,7 +571,7 @@ def pilot_points_to_tpl(pp_file, tpl_file=None, name_prefix=None, longnames=Fals
 
     Example::
 
-        pyemu.pp_utils.pilot_points_to_tpl("my_pps.dat",name_prefix="my_pps",longnames=True)
+        pyemu.pp_utils.pilot_points_to_tpl("my_pps.dat",name_prefix="my_pps")
 
 
     """
@@ -523,86 +582,121 @@ def pilot_points_to_tpl(pp_file, tpl_file=None, name_prefix=None, longnames=Fals
     else:
         assert os.path.exists(pp_file)
         pp_df = pd.read_csv(pp_file, delim_whitespace=True, header=None, names=PP_NAMES)
-
+    pp_df = pp_df.astype({'zone': int}, errors='ignore')
     if tpl_file is None:
         tpl_file = pp_file + ".tpl"
 
-    if longnames:
-        if name_prefix is not None:
-            if "i" in pp_df.columns and "j" in pp_df.columns:
-                pp_df.loc[:, "parnme"] = pp_df.apply(
-                    lambda x: "{0}_i:{1}_j:{2}".format(name_prefix, int(x.i), int(x.j)),
-                    axis=1,
-                )
-            elif "x" in pp_df.columns and "y" in pp_df.columns:
-                pp_df.loc[:, "parnme"] = pp_df.apply(
-                    lambda x: "{0}_x:{1:0.2f}_y:{2:0.2f}".format(name_prefix, x.x, x.y),
-                    axis=1,
-                )
-            else:
-                pp_df.loc[:, "idx"] = np.arange(pp_df.shape[0])
-                pp_df.loc[:, "parnme"] = pp_df.apply(
-                    lambda x: "{0}_ppidx:{1}".format(name_prefix, x.idx),
-                    axis=1,
-                )
-            if "zone" in pp_df.columns:
-                pp_df.loc[:, "parnme"] = pp_df.apply(
-                    lambda x: x.parnme + "_zone:{0}".format(x.zone), axis=1
-                )
-            pp_df.loc[:, "tpl"] = pp_df.parnme.apply(
-                lambda x: "~    {0}    ~".format(x)
+    if name_prefix is not None:
+        if "i" in pp_df.columns and "j" in pp_df.columns:
+            pp_df.loc[:, "parnme"] = pp_df.apply(
+                lambda x: "{0}_i:{1}_j:{2}".format(name_prefix, int(x.i), int(x.j)),
+                axis=1,
+            )
+        elif "x" in pp_df.columns and "y" in pp_df.columns:
+            pp_df.loc[:, "parnme"] = pp_df.apply(
+                lambda x: "{0}_x:{1:0.2f}_y:{2:0.2f}".format(name_prefix, x.x, x.y),
+                axis=1,
             )
         else:
-            names = pp_df.name.copy()
-            pp_df.loc[:, "parnme"] = pp_df.name
-            pp_df.loc[:, "tpl"] = pp_df.parnme.apply(
-                lambda x: "~    {0}    ~".format(x)
+            pp_df.loc[:, "idx"] = np.arange(pp_df.shape[0])
+            pp_df.loc[:, "parnme"] = pp_df.apply(
+                lambda x: "{0}_ppidx:{1}".format(name_prefix, x.idx),
+                axis=1,
             )
-        _write_df_tpl(
-            tpl_file,
-            pp_df.loc[:, ["name", "x", "y", "zone", "tpl"]],
-            sep=" ",
-            index_label="index",
-            header=False,
-            index=False,
-            quotechar=" ",
-            quoting=2,
+        if "zone" in pp_df.columns:
+            pp_df.loc[:, "parnme"] = pp_df.apply(
+                lambda x: x.parnme + "_zone:{0}".format(x.zone), axis=1
+            )
+        pp_df.loc[:, "tpl"] = pp_df.parnme.apply(
+            lambda x: "~    {0}    ~".format(x)
         )
     else:
-        if name_prefix is not None:
-            digits = str(len(str(pp_df.shape[0])))
-            fmt = "{0:0" + digits + "d}"
-            if len(name_prefix) + 1 + int(digits) > 12:
-                warnings.warn("name_prefix too long for parameter names", PyemuWarning)
-            names = [
-                "{0}_{1}".format(name_prefix, fmt.format(i))
-                for i in range(pp_df.shape[0])
-            ]
-        else:
-            names = pp_df.name.copy()
-        too_long = []
-        for name in names:
-            if len(name) > 12:
-                too_long.append(name)
-        if len(too_long) > 0:
-            raise Exception(
-                "the following parameter names are too long:" + ",".join(too_long)
-            )
-        tpl_entries = ["~    {0}    ~".format(name) for name in names]
-        pp_df.loc[:, "tpl"] = tpl_entries
-        pp_df.loc[:, "parnme"] = names
-        f_tpl = open(tpl_file, "w")
-        f_tpl.write("ptf ~\n")
-        f_tpl.write(
-            pp_df.to_string(
-                col_space=0,
-                columns=["name", "x", "y", "zone", "tpl"],
-                formatters=PP_FMT,
-                justify="left",
-                header=False,
-                index=False,
-            )
-            + "\n"
+        pp_df.loc[:, "parnme"] = pp_df.name
+        pp_df.loc[:, "tpl"] = pp_df.parnme.apply(
+            lambda x: "~    {0}    ~".format(x)
         )
+    _write_df_tpl(
+        tpl_file,
+        pp_df.loc[:, ["name", "x", "y", "zone", "tpl"]],
+        sep=" ",
+        index_label="index",
+        header=False,
+        index=False,
+        quotechar=" ",
+        quoting=2,
+    )
 
     return pp_df
+
+def get_zoned_ppoints_for_vertexgrid(spacing, zone_array, mg, zone_number=None, add_buffer=True):
+    """Generate equally spaced pilot points for active area of DISV type MODFLOW6 grid. 
+ 
+    Args:
+        spacing (`float`): spacing in model length units between pilot points. 
+        zone_array (`numpy.ndarray`): the modflow 6 idomain integer array.  This is used to
+            set pilot points only in active areas and to assign zone numbers. 
+        mg  (`flopy.discretization.vertexgrid.VertexGrid`): a VertexGrid flopy discretization dervied type.
+        zone_number (`int`): zone number 
+        add_buffer (`boolean`): specifies whether pilot points ar eplaced wihtin a buffer zone of size `distance` around the zone/active domain
+
+    Returns:
+        `list`: a list of tuples with pilot point x and y coordinates
+
+    Example::
+
+        get_zoned_ppoints_for_vertexgrid(spacing=100, ib=idomain, mg, zone_number=1, add_buffer=False)
+
+    """
+
+    try:
+        from shapely.ops import unary_union
+        from shapely.geometry import Polygon, Point
+    except ImportError:
+        raise ImportError('The `shapely` library was not found. Please make sure it is installed.')
+
+    
+    if mg.grid_type=='vertex' and zone_array is not None and len(zone_array.shape)==1:
+            zone_array = np.reshape(zone_array, (zone_array.shape[0], ))
+
+
+    assert zone_array.shape[0] == mg.xcellcenters.shape[0], "The ib idomain array should be of shape (ncpl,). i.e. For a single layer."
+
+    if zone_number:
+        assert zone_array[zone_array==zone_number].shape[0]>0, f"The zone_number: {zone_number} is not in the ib array."
+
+    # get zone cell x,y 
+    xc, yc = mg.xcellcenters, mg.ycellcenters
+    if zone_number != None:
+        xc_zone = xc[np.where(zone_array==zone_number)[0]]
+        yc_zone = yc[np.where(zone_array==zone_number)[0]]
+    else:
+        xc_zone = xc[np.where(zone_array>0)]
+        yc_zone = yc[np.where(zone_array>0)]   
+
+    # get outer bounds
+    xmin, xmax = min(xc_zone), max(xc_zone)
+    ymin, ymax = min(yc_zone), max(yc_zone)
+    # n-ppoints
+    nx = int(np.ceil((xmax - xmin) / spacing))
+    ny = int(np.ceil((ymax - ymin) / spacing))
+    # make even spaced points
+    x = np.linspace(xmin, xmax, nx)
+    y = np.linspace(ymin, ymax, ny)
+    xv, yv = np.meshgrid(x, y)
+    # make grid
+    grid_points = [Point(x,y) for x,y in list(zip(xv.flatten(), yv.flatten()))]
+
+    # get vertices for model grid/zone polygon
+    verts = [mg.get_cell_vertices(cellid) for cellid in range(mg.ncpl)]
+    if zone_number != None:
+        # select zone area
+        verts = [verts[i] for i in np.where(zone_array==zone_number)[0]]
+    # dissolve
+    polygon = unary_union([Polygon(v) for v in verts])
+    # add buffer
+    if add_buffer==True:
+        polygon = polygon.buffer(spacing)
+    # select ppoint coords within area
+    ppoints = [(p.x, p.y) for p in grid_points if polygon.covers(p) ]
+    assert len(ppoints)>0
+    return ppoints
\ No newline at end of file
